@@ -23,9 +23,10 @@
 * Report bugs to <farmatito@tiscali.it>
 *
 */
- 
+#define _GNU_SOURCE
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <gtk/gtk.h>
@@ -54,6 +55,40 @@ static void wifi_switcher_class_init (WifiSwitcherClass *klass)
 	g_type_class_add_private (klass, sizeof (WifiSwitcherPrivate));
 }
 
+/** Check if wifi module is loaded
+    @return One if wifi module is loaded, zero otherwise
+*/
+static int check_wifi_module_loaded()
+{
+	int module_loaded = 0;
+	char *lineptr = NULL;
+	size_t res_sz;
+	FILE *fp = fopen("/proc/modules", "r");
+
+	if (fp) {
+		while (getline(&lineptr, &res_sz, fp) != -1) {
+			if (strncmp("wl12xx ", lineptr, 7) == 0) {
+				module_loaded = 1;
+				break;
+			}
+		}
+		free(lineptr);
+		fclose (fp);
+	}
+	return module_loaded;
+}
+
+/** Set Wifi button text according to wifi module state
+	@param priv Pointer to WifiSwitcherPrivate structure
+*/
+static void set_wifi_button_text(WifiSwitcherPrivate *priv)
+{
+	const char *subtext = check_wifi_module_loaded() ? "On" : "Off";
+	hildon_button_set_text (HILDON_BUTTON (priv->button),
+				"Wireless LAN",
+				subtext);
+}
+
 static void wifi_switcher_init (WifiSwitcher *plugin)
 {
 	WifiSwitcherPrivate *priv = WIFI_SWITCHER_GET_PRIVATE (plugin);
@@ -64,7 +99,7 @@ static void wifi_switcher_init (WifiSwitcher *plugin)
 
 	hildon_button_set_style (HILDON_BUTTON(priv->button), HILDON_BUTTON_STYLE_PICKER);
 	hildon_button_set_image (HILDON_BUTTON(priv->button), image);
-	hildon_button_set_text (HILDON_BUTTON (priv->button), "Wifi Switcher", NULL);
+	set_wifi_button_text(priv);
 	hildon_button_set_alignment (HILDON_BUTTON(priv->button), 0.0, 0.0, 1.0, 1.0);
 
 	g_signal_connect_after (G_OBJECT (priv->button), "clicked", G_CALLBACK (wifi_switcher_cb), plugin);
@@ -74,9 +109,13 @@ static void wifi_switcher_init (WifiSwitcher *plugin)
 
 static gboolean run(gpointer plugin)
 {
+	WifiSwitcherPrivate *priv = WIFI_SWITCHER_GET_PRIVATE (plugin);
 	int status =  WEXITSTATUS(system("sudo /usr/sbin/wifi.sh"));
 	GdkPixbuf *pixbuf = NULL;
 
+	/* Update wifi button text */
+	set_wifi_button_text(priv);
+
 	if (status == 2) {
 		/* Put the icon at the status bar*/	
 		pixbuf = gdk_pixbuf_new_from_file_at_size ("/usr/share/icons/hicolor/scalable/hildon/wlan_off.png",
